package com.anabas.testsharedlet;

import com.anabas.concepts.*;
import com.anabas.sharedlet.*;
import com.anabas.util.misc.LogManager;
import com.anabas.gxo.*;
import com.anabas.naming.*;

import java.net.URL;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.Enumeration;
import java.util.Vector;

/**
 * This contains the core logic for this test sharedlet.
 *
 * @author John Yin
 * @version 1.0
 */

public class TestControlView extends JPanel implements SharedletView,
                                                       ActionListener,
                                                       MouseListener,
                                                       GMS_MessageListener,
                                                       SessionListener,
                                                       CapabilityListener {
  // Used to generate random number messages
  private static java.util.Random s_random = new java.util.Random(System.currentTimeMillis());

  private SharedletViewRenderer m_renderer;
  private PresentationView      m_presentationView;

  private CapabilitiesManager m_capabilitiesManager;
  private SessionManager      m_sessionManager;
  private CommunicationService m_commService;
  private LauncherService     m_launcherService;
  private GMS_StreamPublisher m_publisher;
  private GMS_StreamSubscriber m_subscriber;

  /* ================= UI Objects ================================ */

  // Button that triggers the generation of a random number.
  private JButton m_actionButton;
  // Shows the user list
  private JComboBox m_userList = new JComboBox();
  // Give the ability to generate random number to a user
  private JButton m_giveButton = new JButton();
  // Take away the ability to generate random number ffrom a user.
  private JButton m_takeButton = new JButton();
  // Promote another user to Moderator.  Allowing that user to
  // remotely take/give random number generation control.
  private JButton m_promoteButton = new JButton();
  // Disallow another user to remotely take/give random number generation control.
  private JButton m_demoteButton = new JButton();
  // Open a file.  Used for testing the loadDocument/document support loop.
  private JButton m_openButton = new JButton();
  // File chooser to choose a file.
  private JFileChooser m_fileChooser = new JFileChooser();

  public TestControlView() {
    // Grab the handle to the communication service and get the handle to the
    // event bus stream for this sharedlet.
    try {
      Context ctx = ContextManager.getInitialContext();
      m_commService = (CommunicationService)ctx.lookup("services/CommunicationService");
      // Sharedlets should use their MIME type or some derivative of it
      // as the name of the stream to avoid namespace collisions.
      // Try to find an existing stream.
      GMS_Stream stream = m_commService.findStream(TestSharedletInfo.g_sharedletMIME);
      // If the stream doesn't exist then create it.
      if (stream == null) {
        stream = m_commService.createStream(TestSharedletInfo.g_sharedletMIME);
      }

      // Get a handle to the launcher server for launch files.
      m_launcherService = (LauncherService)ctx.lookup("services/LauncherService");

      // Create a publisher & subscriber to the stream so it can receive and send
      // messages to the stream.
      m_publisher = stream.createPublisher();
      m_subscriber = stream.createSubscriber();

      // Set itself as the message listener waiting for incoming messages and
      // requests.
      m_subscriber.setMessageListener(this);

      // Get a handle to the capabilities manager so it can check for what permssions
      // this user has.
      m_capabilitiesManager = (CapabilitiesManager)ctx.lookup("services/CapabilitiesManager");
      // Needs to be a listener to know when capabilities change so it can enable/disable
      // certain buttons.
      m_capabilitiesManager.addCapabilityListener(this);

      Vector moderatables = m_capabilitiesManager.getModeratableElements(TestSharedletInfo.g_sharedletMIME);
      Enumeration enum = moderatables.elements();
      while(enum.hasMoreElements()) {
        String me = (String)enum.nextElement();
        LogManager.log("TestSharedlet","Visible Moderatable Element: " + me);
      }

      // Get a handle to the session manager and act as a listener to update the
      // user list selector.
      m_sessionManager = (SessionManager)ctx.lookup("services/SessionManager");
      m_sessionManager.addSessionListener(this);

      // Initialize the user list selector.
      Vector p = m_sessionManager.getParticipants();
      enum = p.elements();
      while(enum.hasMoreElements()) {
        m_userList.addItem(((User)enum.nextElement()).getUserID());
      }
    } catch (Exception e) {
      LogManager.err("TestControlView","Unable to get comm service",e);
    }

    m_renderer = new JavaViewRenderer(this,this);

    try {
      jbInit();
    } catch (Exception e) {
    }
  }

  /**
   * Do nothing.
   */
  public void activate() {
    LogManager.log("View",getID() + " view activated");
  }

  /**
   * Do nothing.
   */
  public void deactivate() {
    LogManager.log("View",getID() + " view deactivated");
  }

  /**
   * Should be triggered by a user clicking on the "Open" button and choosing
   * a file.
   */
  public void loadDocument(java.net.URL documentURL) {
    LogManager.log("View",getID() + " load document: " + documentURL);
  }

  /**
   * @return "application/x-sharedlet-docnav"
   */
  public String getSharedletMIME() {
    return TestSharedletInfo.g_sharedletMIME;
  }

  /**
   * Invoked when this sharedlet's capability manager has changed modes.
   * Not currently used; however, it could alternatively be used to manage
   * the moderator control rather than manually doing it.
   */
  /*public void modeChanged(short newMode) {
    LogManager.log("TestSharedlet","Capabilities manager mode changed to: " + newMode);
  }*/

  /**
   * Invoked when this sharedlet's role has changed.
   */
  public void roleChanged(String newRole) {
    LogManager.log("TestSharedlet","Role changed to : "+newRole);
  }

  /**
   * Listen for remote capability changes.
   */
  public void remoteCapabilityChanged(String capability,boolean enabled,UserID user) {
    LogManager.log("TestControlView","Remote '"+capability+"' is now " +
                   enabled + " for user: " + user);
  }

  /**
   * Listen for capability changes and enable/disable appropriate set of buttons.
   * changes.
   */
  public void capabilityChanged(String capability,boolean enabled) {
    LogManager.log("TestControlView","My '"+capability+"' is now " + enabled);
    if (capability.equals("1:start")) {
      m_actionButton.setEnabled(enabled);
    } else if (capability.equals("give")) {
      m_giveButton.setEnabled(enabled);
    } else if (capability.equals("take")) {
      m_takeButton.setEnabled(enabled);
    } else if (capability.equals("moderate")) {
      m_promoteButton.setEnabled(enabled);
      m_demoteButton.setEnabled(enabled);
    }
  }

  /**
   * Invoked by SessionManager service when a user joins.  Need to
   * update the user list.
   */
  public void userJoined(SessionEvent e) {
    User u = (User)e.getSource();
    m_userList.addItem(u.getUserID());
  }

  /**
   * Invoked by SessionManager service when a user leaves.  Need to
   * update the user list.
   */
  public void userLeft(SessionEvent e) {
    User u = (User)e.getSource();
    m_userList.removeItem(u.getUserID());
  }

  /**
   * When it receives a message from the bus containing a new random number
   * that was generated by another user, display it in the presentation area.
   */
  public void onMessage(GMS_Message m) {
    try {
      GMS_TextMessage msg = (GMS_TextMessage)m;

      // Get a handle to the presentation view if I don't have it already.
      if (m_presentationView == null) {
        initPresentationView();
      }
      m_presentationView.displayMessage(msg.getText());
    } catch (GXO_Exception e) {
      LogManager.err("TestControlView","error on message",e);
    }
  }

  /**
   * Main trap for various button actions.
   */
  public void actionPerformed(ActionEvent e) {
    if (e.getActionCommand().equals("Start")) {
      try {
        // Create a new text message to contain the new random number.
        GMS_TextMessage msg = (GMS_TextMessage)m_publisher.createMessage(GMS_Message.MSGTYPE_MINOR,
                                                      GMS_Message.BODYTYPE_TEXT);

        // Set the new random number into the message.
        msg.setText("Random Double Message: " + Double.toString(s_random.nextDouble()));

        // Broadcast onto the stream.
        m_publisher.broadcast(msg);
        LogManager.log("TestControlView","Broadcasted " + msg.getText());
      } catch (GXO_Exception ex) {
        LogManager.err("TextSharedlet","Unable to broadcast",ex);
      }
    } else if (e.getActionCommand().equals("givecontrol")) {
      try {
        LogManager.log("TestControlView","Enabling capability on moderation to" +
                       ((UserID)m_userList.getSelectedItem()).toString());
        // Use the capabilities manager API to enable a remote user's
        // "Application Control" moderatable element for this sharedlet.
        // The remote capability manager will automatically find the mapping
        // to the list of capabilities and change them thus firing off
        // capability events.
        m_capabilitiesManager.enableRemoteModeratable(TestSharedletInfo.g_sharedletMIME,
                                                      "Application Control",
                                                      (UserID)m_userList.getSelectedItem());
      } catch (PermissionDeniedException ex) {
        LogManager.err("TestControlView","Can not give control, permission denied",ex);
      }
    } else if (e.getActionCommand().equals("takecontrol")) {
      try {
        LogManager.log("TestControlView","Disabling capability on moderation to" +
                       ((UserID)m_userList.getSelectedItem()).toString());
        // Use the capabilities manager API to disable a remote user's
        // "Application Control" moderatable element for this sharedlet.
        // The remote capability manager will automatically find the mapping
        // to the list of capabilities and change them thus firing off
        // capability events.
        m_capabilitiesManager.disableRemoteModeratable(TestSharedletInfo.g_sharedletMIME,
                                             "Application Control",
                                             (UserID)m_userList.getSelectedItem());
      } catch (PermissionDeniedException ex) {
        LogManager.err("TestControlView","Can not take away control, permission denied",ex);
      }
    } else if (e.getActionCommand().equals("promote")) {
      try {
        UserID user = (UserID)m_userList.getSelectedItem();
        String remoteRole = m_capabilitiesManager.getRemoteApplicationRole(user);
        String newRole = "";
        if (remoteRole.equals("host")) {
         LogManager.log("TestControlView","User " + ((UserID)m_userList.getSelectedItem()).toString()+
                        " already host...no further promotion possible");
        } else if (remoteRole.equals("moderator")) {
         LogManager.log("TestControlView","User " + ((UserID)m_userList.getSelectedItem()).toString() +
                        " being promoted to host");
         newRole = "host";
        } else if (remoteRole.equals("participant")) {
         LogManager.log("TestControlView","User " + ((UserID)m_userList.getSelectedItem()).toString() +
                        " being promoted to moderator");
         newRole = "moderator";
        }

        // Promote another user to moderator mode.
        // First we have to set the remote capability manager to Moderator Mode
        // so that it can remotely set other people's capabilities.
        if (newRole.length() > 0) {
          m_capabilitiesManager.setRemoteApplicationRole(user,newRole);
        }
      } catch (PermissionDeniedException ex) {
        LogManager.err("TestControlView","Can not give control, permission denied",ex);
      }
    } else if (e.getActionCommand().equals("demote")) {
      try {
        UserID user = (UserID)m_userList.getSelectedItem();
        String remoteRole = m_capabilitiesManager.getRemoteApplicationRole(user);
        String newRole = "";
        if (remoteRole.equals("host")) {
         LogManager.log("TestControlView","User " + ((UserID)m_userList.getSelectedItem()).toString() +
                        " demoting to moderator");
         newRole = "moderator";
        } else if (remoteRole.equals("moderator")) {
         LogManager.log("TestControlView","User " + ((UserID)m_userList.getSelectedItem()).toString() +
                        " being demoted to participant");
         newRole = "participant";
        } else if (remoteRole.equals("participant")) {
         LogManager.log("TestControlView","User " + ((UserID)m_userList.getSelectedItem()).toString() +
                        " is already the lowest of the low.");
        }

        // Promote another user to moderator mode.
        // First we have to set the remote capability manager to Moderator Mode
        // so that it can remotely set other people's capabilities.
        if (newRole.length() > 0) {
          m_capabilitiesManager.setRemoteApplicationRole(user,newRole);
        }
      } catch (PermissionDeniedException ex) {
        LogManager.err("TestControlView","Can not give control, permission denied",ex);
      }
    } else if (e.getActionCommand().equals("open")) {
      // Opens a file.
      int returnVal = m_fileChooser.showOpenDialog(this);
      if (returnVal == JFileChooser.APPROVE_OPTION) {
        openFile(m_fileChooser.getSelectedFile().getName());
      }
    }
  }

  /**
   * Invokes the LauncherService's launchDocument.  Since this sharedlet supports
   * "*" document types, the TestSharedlet.loadDocument() should get invoked.
   */
  private void openFile(String filename) {
    try {
      m_launcherService.launch(new URL("file","localhost",filename));
    } catch (Exception e) {
      LogManager.err("Control View","Malformed URL: " + filename);
    }
  }

  /**
   * These function are not used.
   */
  public void onRequest(GMS_Message m,GMS_MessageReplyer r) {}
  public void mouseClicked(MouseEvent e) {}
  public void mousePressed(MouseEvent e) {};
  public void mouseReleased(MouseEvent e) {};
  public void mouseEntered(MouseEvent e) {};
  public void mouseExited(MouseEvent e) {};
  public void mouseDragged(MouseEvent e) {};

  /**
   * @return The unique name of the control view.
   */
  public String getID() {
    return "Test Control";
  }

  /**
   * @return The Java renderer
   */
  public SharedletViewRenderer getRenderer(String MIMEType) {
    if (MIMEType.equals("application/java")) {
      return m_renderer;
    }
    return null;
  }

  /**
   * Initializes the control view layout.  Just a simple flow layout.
   */
  private void jbInit() throws Exception {
    setLayout(new FlowLayout());

    Dimension d = new Dimension(150,150);
    this.setPreferredSize(d);

    m_actionButton = new JButton("Start");

    m_giveButton.setActionCommand("givecontrol");
    m_giveButton.setText("Give");
    m_takeButton.setActionCommand("takecontrol");
    m_takeButton.setText("Take");
    m_promoteButton.setActionCommand("promote");
    m_promoteButton.setText("Promote");
    m_demoteButton.setActionCommand("demote");
    m_demoteButton.setText("Demote");
    m_openButton.setActionCommand("open");
    m_openButton.setText("Open");
    add(m_actionButton);
    this.add(m_userList, null);
    this.add(m_giveButton, null);
    this.add(m_takeButton, null);
    this.add(m_promoteButton, null);
    this.add(m_demoteButton, null);
    this.add(m_openButton, null);
    this.addMouseListener(this);

    m_actionButton.addActionListener(this);
    m_giveButton.addActionListener(this);
    m_takeButton.addActionListener(this);
    m_promoteButton.addActionListener(this);
    m_demoteButton.addActionListener(this);
    m_openButton.addActionListener(this);
    m_fileChooser.addActionListener(this);


    if (!m_capabilitiesManager.isCapable(TestSharedletInfo.g_sharedletMIME,"1:start")) {
      m_actionButton.setEnabled(false);
    }

    if (!m_capabilitiesManager.isCapable(TestSharedletInfo.g_sharedletMIME,"take")) {
      m_takeButton.setEnabled(false);
    }

    if (!m_capabilitiesManager.isCapable(TestSharedletInfo.g_sharedletMIME,"give")) {
      m_giveButton.setEnabled(false);
    }

    if (!m_capabilitiesManager.isCapable(TestSharedletInfo.g_sharedletMIME,"moderate")) {
      m_promoteButton.setEnabled(false);
      m_demoteButton.setEnabled(false);
    }
  }

  /**
   * Grabs a handle to the presentation view by looking it up under
   * <root context>/sharedlets/x-sharedlet-docnav/views/com.anabas.TestSharedletInfo.PresentationView
   */
  private void initPresentationView() {
    Context ctx = ContextManager.getInitialContext();

    try {
      m_presentationView = (PresentationView)ctx.lookup("sharedlets/application/x-sharedlet-test/views/com.anabas.testsharedlet.PresentationView");
    } catch (NamingException e) {
      LogManager.err("TestControlView","Unable to get presentation view",e);
    }
  }
}